__all__ = ["ProxyRotator"]

from .ProxyRotator import BaseProxyRotator
from .proxy_storage import ProxyStorage
from .proxy_types import ProxyType
from .schemas import ProxySchema
from typing import Optional
import logging


class ProxyRotator(BaseProxyRotator):
    def __init__(self, proxy_storage: ProxyStorage, logger: logging.Logger) -> None:
        self.__proxy_storage = proxy_storage
        self.__logger = logger 
        
    def rotate(self, old_proxy: Optional[ProxySchema] = None, delete: bool = False, type: ProxyType = ProxyType.HTTP) -> str:
        if delete:
            if old_proxy is None:
                raise ValueError("old_proxy must be not None")
            
            try:
                self.__proxy_storage.delete(old_proxy)
            except ValueError as e:
                self.__logger.error(f"Error deleting old proxy: {e}")
        
        try:
            new_proxy = self.__proxy_storage.get(type)
        except KeyError:
            raise ValueError(f"No proxies available for type [{type}]")

        return f"{new_proxy.ip_address}:{new_proxy.port}"



__all__ = ["ProxyStorage"]

from .ProxyStorage import BaseProxyStorage
from .schemas import ProxySchema
from .proxy_types import ProxyType
from typing import List, Dict
import random
import logging


class ProxyStorage(BaseProxyStorage):
    def __init__(self, logger: logging.Logger) -> None:
        self.__proxy: Dict[ProxyType, List[ProxySchema]] = {}
        self.__deleted: int = 0
        self.__logger = logger

    def add(self, proxy: ProxySchema) -> None:
        if proxy.type not in self.__proxy:
            self.__proxy[proxy.type] = []

        self.__proxy[proxy.type].append(proxy)

    def get(self, type: ProxyType) -> ProxySchema:
        if type not in self.__proxy:
            raise KeyError(f"Key error [{type}]")

        if not self.__proxy[type]:
            raise ValueError(f"No proxies available for type")
        
        return random.choice(self.__proxy[type])

    def delete(self, proxy: ProxySchema) -> None:
        for i, current_proxy in enumerate(self.__proxy[proxy.type]):
            if current_proxy.ip_address == proxy.ip_address and current_proxy.port == proxy.port:
                self.__proxy[proxy.type].pop(i)
                self.__deleted += 1
                self.__logger.info(f"[🗑] {current_proxy.type}://{current_proxy.ip_address}:{current_proxy.port} deleted.")
                return True
        return False
    
    def __str__(self) -> str:
        lines = ["\n"]
        for key in self.__proxy.keys():
            lines.append(f"{key} - {len(self.__proxy[key])}")
        
        lines.append(f"🗑 deleted: {self.__deleted}\n")

        return f"\n".join(lines)



__all__ = ["BaseProxyStorage"]

from abc import ABC, abstractmethod
from .proxy_types import ProxyType
from .schemas import ProxySchema


class BaseProxyStorage(ABC):
    """
    # Abstract class for storing and managing proxies

    This class defines the interface for storing proxies, retrieving proxies by type, and deleting proxies.
    Concrete implementations of this class should provide the actual mechanisms for working with proxy schemas.
    """
    @abstractmethod
    def add(self, proxy: ProxySchema) -> None:
        """
        # Adds a new proxy to the storage.

        :param proxy: The proxy schema containing information about the proxy to be added.
        """
        NotImplementedError()

    @abstractmethod
    def get(self, type: ProxyType) -> ProxySchema:
        """
        # Returns a random proxy

        :param type: The type of proxy to retrieve. This value should match one of the allowed proxy types defined in ProxyType.
        :return: The proxy schema corresponding to the specified type.
        """
        NotImplementedError()

    @abstractmethod
    def delete(self) -> bool:
        """
        # Deletes a proxy from the storage.
        
        :return: True if the proxy was successfully deleted, otherwise False."""
        NotImplementedError()



__all__ = ["ProxySchema"]

from pydantic import BaseModel, IPvAnyAddress, conint
from typing import Annotated
from .proxy_types import ProxyType


class ProxySchema(BaseModel):
    type: ProxyType
    ip_address: IPvAnyAddress
    port: Annotated[int, conint(ge=1024, le=65535)]



from setuptools import setup, find_packages

with open('requirements.txt') as f:
    required = f.read().splitlines()


setup(
    name="dwarf_rotator",
    version="0.1.1",
    author="yeegie",
    long_description=open('readme.md').read(),
    author_email="lotus9200@gmail.com",
    package_dir={"": "src"},
    packages=find_packages(where="src"),
    include_package_data=True,
    description="Proxy helper, store and rotate proxies.",
    install_requires=required,
)



__all__ = ["BaseProxyRotator"]

from abc import ABC, abstractmethod
from .schemas import ProxySchema
from .proxy_types import ProxyType
from typing import Optional


class BaseProxyRotator(ABC):
    @abstractmethod
    def rotate(self, old_proxy: Optional[ProxySchema], delete: bool = False, type: ProxyType = ProxyType.HTTP) -> str:
        """
        # Rotates the given proxy and returns a new proxy.

        :param old_proxy: Current proxy, not required, specified only when delete is True.
        :param delete: A flag indicating whether the old proxy should be deleted after rotation. Default is False.
        :param type: The type of proxy to return. Default is ProxyType.HTTP.
        :return: The new proxy as a string.
        """
        NotImplementedError()
    